/* eslint-disable @typescript-eslint/no-unsafe-assignment */
import { readFile, readdir, access } from "fs/promises";
import path from "path";

import matter from "gray-matter";
import { Client } from "ldapts";
import { serialize } from "next-mdx-remote/serialize";

import { getCurrentTerm } from "@/lib/events";

const EXECS_PATH = path.join("content", "team", "execs");
const fileType = ".md";
const { year, term } = getCurrentTerm();

const execPositions: { [name: string]: number } = {
  presidentcro: 1,
  "vice-president": 2,
  secretary: 3,
  treasurer: 4,
  sysadmin: 5,
};

const positionNames: string[] = [
  "President",
  "Vice President",
  "Assistant Vice President",
  "Treasurer",
  "System Administrator",
];

export interface execMembers {
  name: string;
  position: string;
}

export interface Metadata {
  name: string;
  role: string;
  image: string;
}

export async function getExecNames() {
  if (process.env.USE_LDAP?.toLowerCase() !== "true") {
    return (await readdir(EXECS_PATH))
      .filter((name) => name.endsWith(fileType))
      .map((name) => name.slice(0, -1 * fileType.length));
  }

  const url = "ldap://ldap1.csclub.uwaterloo.ca";
  const searchDN = "ou=People,dc=csclub,dc=uwaterloo,dc=ca";
  const client = new Client({ url });

  let execMembers: execMembers[] = [];
  let formattedExec: string[];

  try {
    await client.bind("", "");
    const { searchEntries } = await client.search(searchDN, {
      scope: "sub",
      filter: `(&(objectClass=member)(term=${(term as string).slice(
        0,
        1
      )}${year}))`,
    });

    execMembers = searchEntries
      .map((item) => {
        return {
          name: item.cn as string,
          position:
            item.position === undefined ? "none" : (item.position as string),
        };
      })
      .filter((item: execMembers) => item.position in execPositions)
      .sort((item1: execMembers, item2: execMembers) => {
        return execPositions[item1.position] - execPositions[item2.position];
      });

    formattedExec = execMembers.map(
      (member) =>
        `0${execPositions[member.position]}-${member.name
          .split(" ")[0]
          .toLowerCase()}-${member.name.split(" ")[1].toLowerCase()}`
    );

    formattedExec = [...formattedExec, "06-codey"];
  } finally {
    await client.unbind();
  }

  return formattedExec;
}

export async function getExec(fileName: string, convert = true) {
  let content, metadata;

  try {
    const raw = await readFile(path.join(EXECS_PATH, `${fileName}${fileType}`));
    ({ content, data: metadata } = matter(raw));

    const image =
      (metadata.image as string | undefined) ??
      (await getMemberImagePath(metadata.name));

    return {
      content: convert ? await serialize(content) : content,
      metadata: { ...metadata, image } as Metadata,
    };
  } catch (err) {
    const firstName =
      fileName.split("-")[1][0].toUpperCase() + fileName.split("-")[1].slice(1);
    const lastName =
      fileName.split("-")[2][0].toUpperCase() + fileName.split("-")[2].slice(1);

    const posOrder = fileName.split("-")[0][1];
    const posName = positionNames[Number(posOrder) - 1];
    ({ content, metadata } = {
      content: "Coming soon!",
      metadata: {
        name: `${firstName} ${lastName}`,
        role: `${posName}`,
      },
    });

    const image = await getMemberImagePath(metadata.name);

    return {
      content: convert ? await serialize(content) : content,
      metadata: { ...metadata, image } as Metadata,
    };
  }
}

async function getImage(imgPath: string) {
  try {
    await access(path.join("public", imgPath));
    return imgPath;
  } catch {
    return undefined;
  }
}

export async function getMemberImagePath(name: string) {
  const imgPath = path.join("images", "team", name.replace(" ", ""));
  const placeholder = path.join(
    "images",
    "team",
    "team-member-placeholder.svg"
  );
  const img =
    (await getImage(imgPath + ".jpg")) ??
    (await getImage(imgPath + ".png")) ??
    (await getImage(imgPath + ".gif")) ??
    (await getImage(imgPath + ".jpeg")) ??
    placeholder;
  return img;
}
